package dsuyasov.recognition.rulesModel.checkers;

import dsuyasov.recognition.rulesModel.MatrixElement;
import dsuyasov.recognition.files.LoadException;
import dsuyasov.recognition.rulesModel.labels.Labels;
import dsuyasov.recognition.ui.helper.panels.cellSettingsPanel.Selecter;
import dsuyasov.recognition.ui.helper.panels.SelectLabelsPanel;
import dsuyasov.recognition.ui.helper.LabelsChanges;
import dsuyasov.recognition.util.XMLUtil;
import dsuyasov.recognition.util.localization.Localization;

import javax.swing.*;
import java.util.List;
import java.util.LinkedList;
import java.awt.*;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * Developed by Suyasov Dmitriy
 */
public class AllLabelsChecker extends LabelChecker implements LabelsChanges {

    public static final String TITLE = "All symbols";
    public static final String LOCALIZATION_TITLE = Localization.get(Localization.ALL_SYMBOLS_TITLE);

    private static final String SYMBOLS_NODE_NAME = "symbols";
    private static final String SYMBOL_NODE_NAME = "symbol";

    private List<Labels> labelsToChecks;
    private Selecter selecter;

    public AllLabelsChecker() {
        this.labelsToChecks = new LinkedList<Labels>();
    }

    public AllLabelsChecker(Node node) throws LoadException {
        this.labelsToChecks = new LinkedList<Labels>();
        List<Node> labelsNode = XMLUtil.getNode(XMLUtil.getSingleNode(node, SYMBOLS_NODE_NAME), SYMBOL_NODE_NAME);
        for (Node childNode : labelsNode) {
            labelsToChecks.add(Labels.newLabelInstance(childNode));
        }
    }

    public boolean outOfBoundsChecker() {
        return false;
    }

    public boolean internalcheck(MatrixElement[][] matrix, int x, int y) {
        for (Object o : labelsToChecks) {
            boolean contain = false;
            for (Object c : matrix[x][y].getLabels()) {
                if (o.equals(c)) {
                    contain = true;
                    addLabelToLabelsRemover(c);
                    break;
                }
            }
            if (!contain)
             return false;
        }
        return true;
    }

    public Color getColor() {
        return Color.BLUE;
    }

    public String getTitle() {
        return TITLE;
    }

    public String getLocalizationTitle() {
        return LOCALIZATION_TITLE;
    }

    public JPanel drawElement(Selecter selecter, JComponent radio) {
        JPanel allLabelsCheckerPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        selecter.getParentPanel().add(allLabelsCheckerPanel);

        allLabelsCheckerPanel.add(radio);

        this.selecter = selecter;
        JPanel labelsPanel = new SelectLabelsPanel(labelsToChecks, this);
        allLabelsCheckerPanel.add(labelsPanel);
        return allLabelsCheckerPanel;
    }

    public void updateLabels(List<Labels> symbols) {
        labelsToChecks = new LinkedList<Labels>(symbols);
        this.selecter.getParentDialog().pack();
    }

    public void drawCell(Graphics g, Rectangle rect) {
        g.setColor(Color.WHITE);
        g.fillRect(rect.x + rect.width / 2, rect.y + 1, rect.width / 2, rect.height / 2);
        g.setColor(Color.BLACK);
        g.drawRect(rect.x + rect.width * 3 / 5, rect.y + 2, rect.width / 3, rect.height / 3);
    }

    public void toXML(Document doc, Node rootElement) {
        super.toXML(doc, rootElement);
        Node symbolsNode = XMLUtil.addNode(doc, rootElement, SYMBOLS_NODE_NAME);
        for (Labels labels : labelsToChecks) {
            labels.toXML(doc, XMLUtil.addNode(doc, symbolsNode, SYMBOL_NODE_NAME));
        }
    }
}
