package dsuyasov.recognition.rulesModel.checkers;

import dsuyasov.recognition.rulesModel.*;
import dsuyasov.recognition.ui.helper.panels.cellSettingsPanel.Selecter;
import dsuyasov.recognition.ui.helper.panels.ColorCellPanel;
import dsuyasov.recognition.ui.helper.RuleCellChanger;
import dsuyasov.recognition.util.XMLUtil;
import dsuyasov.recognition.util.localization.Localization;
import dsuyasov.recognition.files.LoadException;

import javax.swing.*;
import java.awt.*;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * Developed by Suyasov Dmitriy
 */
public class FixedColorChecker extends ColorChecker implements RuleCellChanger {

    private Color color;

    private static final String COLOR_NODE_NAME = "color";

    public FixedColorChecker() {
        this.color = Color.WHITE;
    }

    public FixedColorChecker(Node node) throws LoadException {
        this.color = XMLUtil.getColor(XMLUtil.getSingleNode(node, COLOR_NODE_NAME));
    }

    public static final String TITLE = "Fixed Color";
    public static final String LOCALIZATION_TITLE = Localization.get(Localization.FIXED_COLOR_TITLE);

    public String getTitle() {
        return TITLE;
    }

    public String getLocalizationTitle() {
        return LOCALIZATION_TITLE;
    }

    public JPanel drawElement(Selecter selecter, JComponent radio) {
        JComponent parent = selecter.getParentPanel();
        JPanel mainPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        parent.add(mainPanel);
        mainPanel.add(radio);

        TypeOfColors typeOfColors =
                ((CellularAutomata)Memory.get(Memory.CURRENT_RULE)).getDefaultRuleItem().getCell(0,0).getColorAcceptable();
        ColorCellPanel colorPanel = new ColorCellPanel(typeOfColors, this.color, "", this);
        mainPanel.add(colorPanel);
        return mainPanel;
    }

    public boolean outOfBoundsChecker() {
        return false;
    }

    public boolean internalcheck(MatrixElement[][] matrix, int x, int y) {
        if (matrix[x][y].getColor().equals(this.color)) {
            return true;
        } else
            return false;
    }

    public void drawCell(Graphics g, Rectangle rect) {
        g.setColor(this.color);
        g.fillRect(rect.x, rect.y, rect.width, rect.height);
        g.setColor(Color.BLACK);
        g.drawRect(rect.x, rect.y, rect.width, rect.height);
    }

    public void updateColor(Color color, String label) {
        this.color = color;
    }

    public void toXML(Document doc, Node rootElement) {
        super.toXML(doc, rootElement);
        XMLUtil.setColor(doc, XMLUtil.addNode(doc, rootElement, COLOR_NODE_NAME), color);
    }
}
