package dsuyasov.recognition.rulesModel.checkers;

import dsuyasov.recognition.rulesModel.MatrixElement;
import dsuyasov.recognition.files.LoadException;
import dsuyasov.recognition.rulesModel.generators.LabelRemover;
import dsuyasov.recognition.ui.helper.Drawer;

import java.util.List;
import java.util.LinkedList;
import java.awt.*;

import org.w3c.dom.Node;

/**
 * Developed by Suyasov Dmitriy
 */
public abstract class LabelChecker extends Drawer {

    public static List<Class<? extends Drawer>> labelCheckers;

    static {
        labelCheckers = new LinkedList<Class<? extends Drawer>>();
        labelCheckers.add(AllLabelsChecker.class);
        labelCheckers.add(NoLabelsChecker.class);
        labelCheckers.add(GeneratedLabelChecker.class);
        labelCheckers.add(OneOfTheLabelsChecker.class);
        labelCheckers.add(NoOfLabelsChecker.class);
    }

    public static List<Drawer> getDrawers() {
        List<Drawer> drawers = new LinkedList<Drawer>();
        for (Class<? extends Drawer> labelChecker : labelCheckers) {
            try {
                drawers.add(labelChecker.newInstance());
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
        return drawers;
    }
    
    public static LabelChecker get(Node node) throws LoadException {
        return (LabelChecker) Drawer.newDrawerInstance(node, getDrawers());
    }

    public boolean check(MatrixElement[][] matrix, int x, int y) {
        if (x < 0 || y < 0 || x >= matrix.length || y >=matrix[0].length)
            return outOfBoundsChecker();
        boolean result = internalcheck(matrix, x, y);
        if (!result && labelRemover != null)
            this.labelRemover.clearLabelToRemove();
        return result;
    }

    public abstract boolean outOfBoundsChecker();

    public abstract boolean internalcheck(MatrixElement[][] matrix, int x, int y);

    public abstract Color getColor();

    public static String[] getTitles() {
        List<Drawer> drawers = getDrawers();
        String[] result = new String[drawers.size()];
        for (int iLoop = 0; iLoop < drawers.size(); iLoop++) {
            Drawer drawer = drawers.get(iLoop);
            result[iLoop] = drawer.getLocalizationTitle();
        }
        return result;
    }
    public static LabelChecker getLabelCheckerByTitle(String title) {
        for (Drawer drawer : getDrawers()) {
            if (drawer.getLocalizationTitle().equals(title)) {
                return (LabelChecker) drawer;
            }
        }
        return null;
    }

    protected LabelRemover labelRemover = null;

    public void addListenerForRemoving(LabelRemover labelRemover) {
        this.labelRemover = labelRemover;
    }
    public void removeListenerForRemoving() {
        this.labelRemover = null;
    }
    public void addLabelToLabelsRemover(Object label) {
        if (this.labelRemover != null)
            this.labelRemover.addLabelToRemove(label);
    }
}
