package dsuyasov.recognition.rulesModel.checkers;

import dsuyasov.recognition.rulesModel.*;
import dsuyasov.recognition.ui.helper.panels.cellSettingsPanel.Selecter;
import dsuyasov.recognition.ui.helper.panels.ColorCellPanel;
import dsuyasov.recognition.ui.helper.RuleCellChanger;
import dsuyasov.recognition.util.XMLUtil;
import dsuyasov.recognition.util.localization.Localization;
import dsuyasov.recognition.files.LoadException;

import javax.swing.*;
import java.awt.*;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * Developed by Suyasov Dmitriy
 */
public class LighterThenColor extends ColorChecker implements RuleCellChanger {

    private int colorR;
    private int colorG;
    private int colorB;
    private int colorA;

    private static final String COLOR_NODE_NAME = "color";

    public LighterThenColor() {
        setColor(Color.WHITE);
    }

    public LighterThenColor(Node node) throws LoadException {
        setColor(XMLUtil.getColor(XMLUtil.getSingleNode(node, COLOR_NODE_NAME)));
    }

    private void setColor(Color color) {
        this.colorR = color.getRed();
        this.colorG = color.getGreen();
        this.colorB = color.getBlue();
        this.colorA = color.getAlpha();
    }

    private Color getColor() {
        return new Color(this.colorR, this.colorG, this.colorB, this.colorA);
    }

    public static final String TITLE = "Lighter Then";
    public static final String LOCALIZATION_TITLE = Localization.get(Localization.LIGHTER_THEN_TITLE);

    public String getTitle() {
        return TITLE;
    }

    public String getLocalizationTitle() {
        return LOCALIZATION_TITLE;
    }

    public JPanel drawElement(Selecter selecter, JComponent radio) {
        JComponent parent = selecter.getParentPanel();
        JPanel mainPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        parent.add(mainPanel);
        mainPanel.add(radio);

        TypeOfColors typeOfColors =
                ((CellularAutomata) Memory.get(Memory.CURRENT_RULE)).getDefaultRuleItem().getCell(0,0).getColorAcceptable();
        ColorCellPanel colorPanel = new ColorCellPanel(typeOfColors, getColor(), "", this);
        mainPanel.add(colorPanel);
        return mainPanel;
    }

    public boolean outOfBoundsChecker() {
        return false;
    }

    public boolean internalcheck(MatrixElement[][] matrix, int x, int y) {
        Color cellColor = matrix[x][y].getColor();
        int cellRed = cellColor.getRed();
        int cellGreen = cellColor.getGreen();
        int cellBlue = cellColor.getBlue();
        int cellAlpha = cellColor.getAlpha();
        return cellRed >= colorR && cellBlue >= colorB && cellGreen >= colorG;
    }

    public void drawCell(Graphics g, Rectangle rect) {
        g.setColor(Color.WHITE);
        int[] xPointsSecond = new int[3];
        xPointsSecond[0] = rect.x;
        xPointsSecond[1] = rect.x + rect.width;
        xPointsSecond[2] = rect.x;

        int[] yPointsSecond = new int[3];
        yPointsSecond[0] = rect.y;
        yPointsSecond[1] = rect.y;
        yPointsSecond[2] = rect.y + rect.height;
        g.fillPolygon(xPointsSecond, yPointsSecond, 3);

        g.setColor(getColor());
        int[] xPoints = new int[3];
        xPoints[0] = rect.x;
        xPoints[1] = rect.x + rect.width;
        xPoints[2] = rect.x + rect.width;

        int[] yPoints = new int[3];
        yPoints[0] = rect.y + rect.height;
        yPoints[1] = rect.y + rect.height;
        yPoints[2] = rect.y;
        g.fillPolygon(xPoints, yPoints, 3);
        g.setColor(Color.BLACK);
        g.drawRect(rect.x, rect.y, rect.width, rect.height);
    }

    public void updateColor(Color color, String label) {
        setColor(color);
    }

    public void toXML(Document doc, Node rootElement) {
        super.toXML(doc, rootElement);
        XMLUtil.setColor(doc, XMLUtil.addNode(doc, rootElement, COLOR_NODE_NAME), getColor());
    }
}
