package dsuyasov.recognition.rulesModel.recognition;

import java.util.*;

/**
 * @author Dmitriy Suyasov
 *         Date 24.05.2007
 */
public class AnalizeResult implements Comparable<AnalizeResult> {

    private String label;
    private double persent;

    public AnalizeResult(String label, double persent) {
        this.label = label;
        this.persent = persent;
    }

    public String getLabel() {
        return label;
    }

    public double getPersent() {
        return persent;
    }

    public AnalizeResult updatePersent(double newPersent) {
        persent = (persent + newPersent) / 2;
        return this;
    }

    public AnalizeResult maxPersent(double newPersent) {
        persent = Math.max(persent, newPersent);
        return this;
    }

    public static void update(List<AnalizeResult> analizeResults, List<StudiesElement> elements, double persent, int number, int x, int y, int l, int correctNumber) {
        for (StudiesElement element : elements) {
            boolean contain = false;
            double mult = 1;
            for (AnalizeResult analizeResult : analizeResults) {
                if (analizeResult.getLabel().equals(element.getLabel())) {
                    analizeResult.maxPersent(persent);
                    contain = true;
                    break;
                }
            }
            if (!contain) {
                analizeResults.add(new AnalizeResult(element.getLabel(), persent));
            }
        }
    }

    public static List<AnalizeResult> merge(List<AnalizeResult> analizeResults1, List<AnalizeResult> analizeResults2) {
        List<AnalizeResult> answer = new LinkedList<AnalizeResult>();
        Map<String, AnalizeResult> map1 = new HashMap<String, AnalizeResult>();
        for (AnalizeResult analizeResult : analizeResults1) {
            map1.put(analizeResult.getLabel(), analizeResult);
        }
        for (AnalizeResult analizeResult : analizeResults2) {
            AnalizeResult mapElement = map1.get(analizeResult.getLabel());
            if (mapElement != null) {
                answer.add(analizeResult.updatePersent(analizeResult.getPersent()));
                map1.remove(analizeResult.getLabel());
            } else {
                answer.add(analizeResult.updatePersent(analizeResult.getPersent() / 3));
            }
        }
        for (String label : map1.keySet()) {
            answer.add(map1.get(label).updatePersent((map1.get(label).getPersent() / 3)));
        }
        return answer;
    }

    public int compareTo(AnalizeResult o) {
        return new Double(o.getPersent()).compareTo(new Double(getPersent()));
    }

    public static class ByLabelComparator implements Comparator<AnalizeResult> {
        public int compare(AnalizeResult o1, AnalizeResult o2) {
            return o1.getLabel().compareTo(o2.getLabel());
        }
    }

}
