package dsuyasov.recognition.rulesModel.recognition;

import dsuyasov.recognition.files.XMLTransferable;
import dsuyasov.recognition.files.LoadException;
import dsuyasov.recognition.util.XMLUtil;

import java.util.*;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * @author Dmitriy Suyasov
 *         Date 24.05.2007
 */
public class StudiesElement implements XMLTransferable {

    private String label;
    private static final String SYMBOL_ATTR_NAME = "symbol";
    private static final String LOCATION_NODE_NAME = "location";

    private Map<Integer, Map<Integer, Map<Integer, Integer>>> numbers;
    private static final String X_ATTR_NAME = "x";
    private static final String Y_ATTR_NAME = "y";
    private static final String L_ATTR_NAME = "l";
    private static final String NUMBERS_ATTR_NAME = "numbers";

    public StudiesElement(String label) {
        this.label = label;
        numbers = new HashMap<Integer, Map<Integer, Map<Integer, Integer>>>();
    }

    public StudiesElement(Node node) throws LoadException {
        label = XMLUtil.getAttribute(node, SYMBOL_ATTR_NAME);
        List<Node> locationNodes = XMLUtil.getNode(node, LOCATION_NODE_NAME);
        numbers = new HashMap<Integer, Map<Integer, Map<Integer, Integer>>>();
        for (Node locationNode : locationNodes) {
            setNumbers(XMLUtil.getIntAttribute(locationNode, L_ATTR_NAME), XMLUtil.getIntAttribute(locationNode, X_ATTR_NAME),
                    XMLUtil.getIntAttribute(locationNode, Y_ATTR_NAME), XMLUtil.getIntAttribute(locationNode, NUMBERS_ATTR_NAME));
        }
    }

    public void setNumbers(int l, int x, int y, int number) {
        Map<Integer, Map<Integer, Integer>> numbers2 = numbers.get(l);
        if (numbers2 == null) {
            numbers2 = new HashMap<Integer, Map<Integer, Integer>>();
            numbers.put(l, numbers2);
        }
        Map<Integer, Integer> numbers3 = numbers2.get(x);
        if (numbers3 == null) {
            numbers3 = new HashMap<Integer, Integer>();
            numbers2.put(x, numbers3);
        }
        numbers3.put(y, number);
    }

    public int getNumbers(int l, int x, int y) {
        return getLayer(l, x, y);
    }

    private int getLayer(int l, int x, int y) {
        Map<Integer, Map<Integer, Integer>> numbers2 = numbers.get(l);
        return numbers2 == null ? 0 : getX(numbers2, x, y);
    }
    private int getX(Map<Integer, Map<Integer, Integer>> numbers2, int x, int y) {
        Map<Integer, Integer> number3 = numbers2.get(x);
        return number3 == null ? 0 : getY(number3, y);
    }
    private int getY(Map<Integer, Integer> numbers3, int y) {
        Integer number4 = numbers3.get(y);
        return number4 == null ? 0 : number4;
    }

    public String getLabel() {
        return label;
    }

    public void toXML(Document doc, Node rootElement) {
        XMLUtil.addAttribute(doc, rootElement, SYMBOL_ATTR_NAME, label);
        for (Integer l : numbers.keySet()) {
            for (Integer x : numbers.get(l).keySet()) {
                for (Integer y : numbers.get(l).get(x).keySet()) {
                    Node node = XMLUtil.addNode(doc, rootElement, LOCATION_NODE_NAME);
                    XMLUtil.addAttribute(doc, node, X_ATTR_NAME, x.toString());
                    XMLUtil.addAttribute(doc, node, Y_ATTR_NAME, y.toString());
                    XMLUtil.addAttribute(doc, node, L_ATTR_NAME, l.toString());
                    XMLUtil.addAttribute(doc, node, NUMBERS_ATTR_NAME, String.valueOf(numbers.get(l).get(x).get(y).toString()));
                }
            }
        }
    }

}
