package dsuyasov.recognition.rulesModel.recognition;

import org.w3c.dom.Node;
import org.w3c.dom.Document;

import java.util.*;

import dsuyasov.recognition.files.XMLTransferable;
import dsuyasov.recognition.files.LoadException;
import dsuyasov.recognition.rulesModel.MatrixElement;
import dsuyasov.recognition.rulesModel.labels.Labels;
import dsuyasov.recognition.util.XMLUtil;

/**
 * @author Dmitriy Suyasov
 *         Date 24.05.2007
 */
public class StudiesSet implements XMLTransferable {

    private List<StudiesElement> elements;

    private Map<Integer, Map<Integer, Map<Integer, List<StudiesElement>>>> index;

    private static final String ELEMENT_NODE_NAME = "element";
    private static final String INDEX_ATTR_NAME = "index";
    private static final String INDEX_ELEMENT_NODE_NAME = "indexElement";

    private static final String X_ATTR_NAME = "x";
    private static final String Y_ATTR_NAME = "y";
    private static final String L_ATTR_NAME = "l";
    private static final String ELEMENT_ATTR_NAME = "element";

    private int x;
    private int y;

    public StudiesSet(int x, int y) {
        elements = new LinkedList<StudiesElement>();
        index = new HashMap<Integer, Map<Integer, Map<Integer, List<StudiesElement>>>>();
        this.x = x;
        this.y = y;
    }

    public int getY() {
        return y;
    }

    public int getX() {
        return x;
    }

    public StudiesSet(Node node) throws LoadException {
        x = XMLUtil.getIntAttribute(node, X_ATTR_NAME);
        y = XMLUtil.getIntAttribute(node, Y_ATTR_NAME);
        List<Node> elementsNodes = XMLUtil.getNode(node, ELEMENT_NODE_NAME);
        elements = new LinkedList<StudiesElement>();
        for (int iLoop = 0; iLoop < elementsNodes.size(); iLoop++) {
            for (Node elementsNode : elementsNodes) {
                if (iLoop == XMLUtil.getIntAttribute(elementsNode, INDEX_ATTR_NAME)) {
                    elements.add(new StudiesElement(elementsNode));
                }
            }
        }
        index = new HashMap<Integer, Map<Integer, Map<Integer, List<StudiesElement>>>>();
        List<Node> elementsIndexes = XMLUtil.getNode(node, INDEX_ELEMENT_NODE_NAME);
        for (Node elementsIndex : elementsIndexes) {
            addToIndex(XMLUtil.getIntAttribute(elementsIndex,  L_ATTR_NAME), XMLUtil.getIntAttribute(elementsIndex, X_ATTR_NAME),
                    XMLUtil.getIntAttribute(elementsIndex, Y_ATTR_NAME), elements.get(XMLUtil.getIntAttribute(elementsIndex, ELEMENT_ATTR_NAME)));
        }
    }

    public int[][][] getNumbers(MatrixElement[][] matrix, List<Labels> acceptLabels) {
        int[][][] numbers = new int[x][y][3];
        for (int iLoop1 = 0; iLoop1 < x; iLoop1++ ) {
            for (int iLoop2 = 0; iLoop2 < y; iLoop2++ ) {
                numbers[iLoop1][iLoop2][0] = 0;
                numbers[iLoop1][iLoop2][1] = 0;
                numbers[iLoop1][iLoop2][2] = 0;
                for (int xLoop = iLoop1 * matrix.length / x; xLoop < (iLoop1 + 1) * matrix.length / x; xLoop++)
                    for (int yLoop = iLoop2 * matrix[0].length / y; yLoop < (iLoop2 + 1) * matrix[0].length / y; yLoop++) {
                        List<Object> labels = matrix[xLoop][yLoop].getLabels();
                        numbers[iLoop1][iLoop2][0] += extractLabelsNumber(acceptLabels, labels, 0);
                        numbers[iLoop1][iLoop2][1] += extractLabelsNumber(acceptLabels, labels, 1);
                        numbers[iLoop1][iLoop2][2] += extractLabelsNumber(acceptLabels, labels, 2);
                    }
            }
        }
        return numbers;
    }

    private int extractLabelsNumber(List<Labels> acceptLabels, List<Object> labels, int level) {
        Labels currentLabel = level >= acceptLabels.size() ? null : acceptLabels.get(level);
        for (Object label : labels) {
            if (label.equals(currentLabel))
                return 1;
        }
        return 0;
    }

    public void study(MatrixElement[][] matrix, List<Labels> acceptLabels, String text) {
        study(getNumbers(matrix, acceptLabels), text);
    }

    public void study(int[][][] numbers, String label) {
        StudiesElement studiesElement = new StudiesElement(label);
        for (int x = 0; x < numbers.length; x++)
            for (int y = 0; y < numbers[0].length; y++)
                for (int layer = 0; layer < numbers[0][0].length; layer++) {
                    if (numbers[x][y][layer] > 0) {
                        studiesElement.setNumbers(layer, x, y, numbers[x][y][layer]);
                        addToIndex(layer, x, y, studiesElement);
                    }
                }
        elements.add(studiesElement);
    }

    private void addToIndex(int l, int x, int y, StudiesElement studiesElement) {
        Map<Integer, Map<Integer, List<StudiesElement>>> numbers2 = index.get(l);
        if (numbers2 == null) {
            numbers2 = new HashMap<Integer, Map<Integer, List<StudiesElement>>>();
            index.put(l, numbers2);
        }
        Map<Integer, List<StudiesElement>> numbers3 = numbers2.get(x);
        if (numbers3 == null) {
            numbers3 = new HashMap<Integer, List<StudiesElement>>();
            numbers2.put(x, numbers3);
        }
        List<StudiesElement> list = numbers3.get(y);
        if (list == null) {
            list = new LinkedList<StudiesElement>();
            numbers3.put(y, list);
        }
        list.add(studiesElement);
    }

    public List<AnalizeResult> analize(MatrixElement[][] matrix, List<Labels> acceptLabels) {
        return analize(getNumbers(matrix, acceptLabels));
    }

    public List<AnalizeResult> analize(int[][][] numbers) {
        List<AnalizeResult> list = analize(numbers, 0);
        correctResultsList(numbers, 0, list);
        List<AnalizeResult> list2 = analize(numbers, 1);
        correctResultsList(numbers, 1, list2);
        list = AnalizeResult.merge(list, list2);
        List<AnalizeResult> list3 = analize(numbers, 2);
        correctResultsList(numbers, 2, list3);
        list = AnalizeResult.merge(list, list3);
        Collections.sort(list);
        return list;
    }

    private void correctResultsList(int[][][] numbers, int layer, List<AnalizeResult> list) {
        for (AnalizeResult analizeResult : list) {
            String label = analizeResult.getLabel();
            for (StudiesElement element : elements) {
                if (element.getLabel().equals(label)) {
                    for (int x = 0; x < numbers.length; x++) {
                        for (int y = 0; y < numbers[0].length; y++) {
                            int number = numbers[x][y][layer];
                            int elNumber = element.getNumbers(layer, x, y);
                            if (number > 0 && elNumber == 0 || number == 0 && elNumber > 0)
                                analizeResult.updatePersent(analizeResult.getPersent() / 2);
                        }
                    }
                }
            }
        }
    }

    public List<AnalizeResult> analize(int[][][] numbers, int layer) {
        List<AnalizeResult> result = new LinkedList<AnalizeResult>();

        for (int x = 0; x < numbers.length; x++) {
            for (int y = 0; y < numbers[x].length; y++) {
                int number = numbers[x][y][layer];
                int correctNumber = 0;
                if (number > 0) {
                    LinkedList<AnalizeResult> localResult = new LinkedList<AnalizeResult>();
                    for (int xx = Math.max(x - 1, 0); xx < Math.min(x + 2, this.x); xx++) {
                        for (int yy = Math.max(y - 1, 0); yy < Math.min(y + 2, this.y); yy++) {
                            double persent = (xx == x && yy == y) ? 100 : 50;
                            List<StudiesElement> list = getFromIndex(layer, xx, yy);
                            if (list != null) AnalizeResult.update(localResult, list, persent, number, xx, yy, layer, correctNumber);
                        }
                    }
                    result = AnalizeResult.merge(result, localResult);
                    correctNumber++;
                }
            }
        }

        return result;
    }

    private List<StudiesElement> getFromIndex(int layer, int x, int y) {
        try {
            return index.get(layer).get(x).get(y);
        } catch(NullPointerException ne) {
            return null;
        }
    }

    private int middle(int a1, int a2) {
        return (a1 + a2) / 2;
    }

    public void toXML(Document doc, Node rootElement) {
        XMLUtil.addAttribute(doc, rootElement, X_ATTR_NAME, String.valueOf(x));
        XMLUtil.addAttribute(doc, rootElement, Y_ATTR_NAME, String.valueOf(y));
        for (int iLoop = 0; iLoop < elements.size(); iLoop++) {
            StudiesElement studiesElement = elements.get(iLoop);
            Node elementNode = XMLUtil.addNode(doc, rootElement, ELEMENT_NODE_NAME);
            XMLUtil.addAttribute(doc, elementNode, INDEX_ATTR_NAME, String.valueOf(iLoop));
            studiesElement.toXML(doc, elementNode);
        }
        for (Integer l : index.keySet()) {
            for (Integer x : index.get(l).keySet()) {
                for (Integer y : index.get(l).get(x).keySet()) {
                    for (StudiesElement studiesElement : index.get(l).get(x).get(y)) {
                        Node indexElementNode = XMLUtil.addNode(doc, rootElement, INDEX_ELEMENT_NODE_NAME);
                        XMLUtil.addAttribute(doc, indexElementNode, L_ATTR_NAME, l.toString());
                        XMLUtil.addAttribute(doc, indexElementNode, X_ATTR_NAME, x.toString());
                        XMLUtil.addAttribute(doc, indexElementNode, Y_ATTR_NAME, y.toString());
                        XMLUtil.addAttribute(doc, indexElementNode, ELEMENT_ATTR_NAME, String.valueOf(elements.indexOf(studiesElement)));
                    }
                }
            }
        }
    }
}
