package dsuyasov.recognition.rulesModel.sequence;

import org.w3c.dom.Node;
import org.w3c.dom.Document;

import javax.swing.*;
import java.awt.*;

import dsuyasov.recognition.util.XMLUtil;
import dsuyasov.recognition.util.localization.Localization;
import dsuyasov.recognition.rulesModel.MatrixElement;
import dsuyasov.recognition.rulesModel.ProcessResult;

/**
 * @author Dmitriy Suyasov
 *         Date 17.05.2007
 */
public class LabelElement extends SpecialElement implements MatrixChangeListener{

    public static final String TITLE = "Label";
    public static final String LOCALIZATION_TITLE = Localization.get(Localization.LABEL_TITLE);

    private static final String LABEL_NAME_NODE_NAME = "labelName"; 

    private String labelName;
    private JTextField labelNameField;
    private boolean change;

    public LabelElement() {
        labelName = "";
        change = false;
    }

    public LabelElement(LabelElement labelElement, AutomataSequence automataSequence) {
        labelName = labelElement.getLabelName();
        change = false;
    }

    public LabelElement(Node node) {
        labelName = XMLUtil.getAttribute(node, LABEL_NAME_NODE_NAME);
        labelName = labelName == null ? "" : labelName;
        change = false;
    }

    public String getLabelName() {
        return labelName;
    }

    public String getTitle() {
        return TITLE;
    }

    public String getLocalizationTitle() {
        return LOCALIZATION_TITLE;
    }

    public void draw(Graphics g, Rectangle rect, int xTextOffset, int yTextOffset) {
        g.setColor(Color.BLACK);
        g.drawLine(rect.x,  rect.y, rect.x, rect.y + rect.height);
        g.drawLine(rect.x + 3,  rect.y, rect.x + 3, rect.y + rect.height);
        g.drawLine(rect.x + 6,  rect.y, rect.x + 6, rect.y + rect.height);

        g.drawLine(rect.x + rect.width,  rect.y, rect.x + rect.width, rect.y + rect.height);
        g.drawLine(rect.x + rect.width - 3,  rect.y, rect.x + rect.width - 3, rect.y + rect.height);
        g.drawLine(rect.x + rect.width - 6,  rect.y, rect.x + rect.width - 6, rect.y + rect.height);

        super.draw(g, rect, rect.x + 9, yTextOffset);
    }

    public String getDescription() {
        return Localization.get(Localization.LABEL_DESCRIPTION); // TODO make description
    }

    protected void drawSettingsFromCleanList(JDialog dialog, JPanel panel, AutomataSequence automataSequence) {
        JPanel labelNamePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        panel.add(labelNamePanel);

        labelNamePanel.add(new JLabel(Localization.get(Localization.LABEL_NAME_TITLE)));
        labelNameField = new JTextField(labelName);
        labelNameField.setColumns(10);
        labelNamePanel.add(labelNameField);
    }

    public boolean confirmCorrectness() {
        labelName = labelNameField.getText();
        return true;
    }

    public String getErrorMessage() {
        return "";
    }

    public boolean canBeRemoved() {
        AutomataSequence automataSequence = AutomataSequence.getRulesSequence();
        int position = -1;
        for (int iLoop = 0; iLoop < automataSequence.getRules().size(); iLoop++) {
            SequenceDisplayable sequenceDisplayable = automataSequence.getRules().get(iLoop);
            if (sequenceDisplayable == this) {
                position = iLoop;
            }
        }
        for (SequenceDisplayable sequenceDisplayable : automataSequence.getRules()) {
            if (sequenceDisplayable instanceof AnyChangeConditionElement &&
                    ((AnyChangeConditionElement) sequenceDisplayable).getSelectedIndex() == position) {
                return false;
            }
        }
        return true;
    }

    public void updateInformation(AutomataSequence automataSequence) {
    }

    public void toXML(Document doc, Node rootElement) {
        super.toXML(doc, rootElement);
        XMLUtil.addAttribute(doc, rootElement, LABEL_NAME_NODE_NAME, labelName);
    }

    public void initialize(AutomataSequence automataSequence) {
        // For AnyChange
        
    }

    public boolean isChange() {
        return change;
    }

    public ProcessResult process(MatrixElement[][] source) {
        change = false;
        return new ProcessResult(source);
    }

    public void change() {
        this.change = true;
    }
}
