package dsuyasov.recognition.rulesModel.sequence;

import dsuyasov.recognition.rulesModel.ProcessResult;
import dsuyasov.recognition.rulesModel.MatrixElement;
import dsuyasov.recognition.files.LoadException;
import dsuyasov.recognition.rulesModel.labels.Labels;
import dsuyasov.recognition.ui.helper.panels.SelectLabelsPanel;
import dsuyasov.recognition.ui.helper.LabelsChanges;
import dsuyasov.recognition.util.XMLUtil;
import dsuyasov.recognition.util.localization.Localization;

import javax.swing.*;
import java.awt.*;
import java.util.List;
import java.util.LinkedList;

import org.w3c.dom.Node;
import org.w3c.dom.Document;

/**
 * @author Dmitriy Suyasov
 */
public class MarkTopLeftElement extends SpecialElement implements LabelsChanges {

    public static final String TITLE = "Mark Top Left Element";
    public static final String LOCALIZATION_TITLE = Localization.get(Localization.MARK_TOP_LEFT_ELEMENT_TITLE);

    public static final String SYMBOL_NODE_NAME = "symbol";

    private JDialog dialog;
    private List<Labels> labels;

    public MarkTopLeftElement() {
        this.labels = new LinkedList<Labels>();
    }

    public MarkTopLeftElement(Node node) throws LoadException {
        this.labels = new LinkedList<Labels>();
        List<Node> labelNodes = XMLUtil.getNode(node, SYMBOL_NODE_NAME);
        for (Node labelNode : labelNodes) {
            this.labels.add(Labels.newLabelInstance(labelNode));
        }
    }

    public MarkTopLeftElement(MarkTopLeftElement markTopLeftElement, AutomataSequence automataSequence) {
        this.labels = markTopLeftElement.getLabels();
    }

    public List<Labels> getLabels() {
        return labels;
    }

    public String getTitle() {
        return TITLE;
    }

    public String getLocalizationTitle() {
        return LOCALIZATION_TITLE;
    }

    protected void drawSettingsFromCleanList(JDialog dialog, JPanel panel, AutomataSequence automataSequence) {
        this.dialog = dialog;
        JPanel symbolsSettingsPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        panel.add(symbolsSettingsPanel);

        symbolsSettingsPanel.add(new JLabel(Localization.get(Localization.LABELS_FOR_MARKING_TITLE)));
        JPanel symbolsPanel = new SelectLabelsPanel(labels, this);
        symbolsSettingsPanel.add(symbolsPanel);
    }

    public boolean confirmCorrectness() {
        return labels != null && labels.size() > 0;
    }

    public String getErrorMessage() {
        return Localization.get(Localization.ONE_SYMBOL_SHOULD_BE_SPECIFIED_MESSAGE);
    }

    public boolean canBeRemoved() {
        return true;
    }

    public void updateInformation(AutomataSequence automataSequence) {}

    public void initialize(AutomataSequence automataSequence) {
    }

    public ProcessResult process(MatrixElement[][] source) {
        for (int y = 0; y < source[0].length; y++) {
            for (int x = 0; x < source.length; x++) {
                if (source[x][y].getColor().equals(Color.BLACK)) {
                    for (Labels label : this.labels) {
                        source[x][y].addLabel(label);
                        source[x][y].makeActions();
                    }
                    return new ProcessResult(source);
                }
            }
        }
        return new ProcessResult(source);
    }

    public void updateLabels(List<Labels> labels) {
        this.labels = new LinkedList<Labels>(labels);
        dialog.pack();
    }

    public void toXML(Document doc, Node rootElement) {
        super.toXML(doc, rootElement);
        for (Labels label : labels) {
            label.toXML(doc, XMLUtil.addNode(doc, rootElement, SYMBOL_NODE_NAME));
        }
    }

    public void draw(Graphics g, Rectangle rect, int xTextOffset, int yTextOffset) {
        g.setColor(Color.BLACK);
        g.drawRect(rect.x, rect.y + 2, rect.width - 2, rect.height - 2);
        g.drawRect(rect.x + rect.width - 10, rect.y, 10, 10);
        super.draw(g, rect, xTextOffset, yTextOffset);
    }

    public String getDescription() {
        return Localization.get(Localization.MARKING_DESCRIPTION);
    }
}
