package dsuyasov.recognition.ui.helper;

import dsuyasov.recognition.ui.helper.panels.cellSettingsPanel.Selecter;
import dsuyasov.recognition.files.XMLTransferable;
import dsuyasov.recognition.files.LoadException;
import dsuyasov.recognition.util.XMLUtil;
import dsuyasov.recognition.util.localization.Localization;

import javax.swing.*;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * Developed by Suyasov Dmitriy
 */
public abstract class Drawer implements XMLTransferable {

    protected static final String TYPE_ATTR_NAME = "type";

    public void draw(Selecter parent) {
        draw(parent, true);
    }

    public JPanel draw(Selecter parent, boolean withRadio) {
        JPanel result;
        if (withRadio) {
            JRadioButton radio = new JRadioButton(getLocalizationTitle());
            radio.addActionListener(new CheckListener(parent, this));
            result = drawElement(parent, radio);
            if (parent.getDefault() != null) {
                if (parent.getDefault().getTitle().equals(getTitle())) {
                    radio.setSelected(true);
                }
            } else if (!parent.getButtonGroup().getElements().hasMoreElements()) {
                radio.setSelected(true);
            }
            parent.getButtonGroup().add(radio);
        } else
            result = drawElement(parent, new JLabel(getLocalizationTitle()));
        return result;
    }

    public abstract String getTitle();
    public abstract String getLocalizationTitle();

    public abstract JPanel drawElement(Selecter selecter, JComponent radio);

    public abstract void drawCell(Graphics g, Rectangle rect);

    public void toXML(Document doc, Node rootElement) {
        XMLUtil.addAttribute(doc, rootElement, TYPE_ATTR_NAME, getTitle());
    }

    public static Drawer newDrawerInstance(Node node, List<Drawer> drawers) throws LoadException {
        String type = XMLUtil.getRequiredAttribute(node, TYPE_ATTR_NAME);
        Drawer newDrawer = null;
        for (Drawer drawer : drawers) {
            if (drawer.getTitle().equals(type)) {
                try {
                    newDrawer = drawer.getClass().getConstructor(Node.class).newInstance(node);
                } catch (Exception e) {
                    throw new LoadException(Localization.get(Localization.PROBLEM_WHILE_INIT_DRAWER_EXCEPTION));
                }
            }
        }
        if (newDrawer == null)
            throw new LoadException(Localization.get(Localization.DRAWER_TYPE_IS_UNKNOWN_EXCEPTION) + " [" + type + "]");
        return newDrawer;
    }

    private class CheckListener implements ActionListener {

        private Selecter selecter;
        private Drawer drawer;

        public CheckListener(Selecter selecter, Drawer drawer) {
            this.selecter = selecter;
            this.drawer = drawer;
        }

        public void actionPerformed(ActionEvent e) {
            this.selecter.setResult(this.drawer);
        }
    }

}
