package dsuyasov.recognition.ui.helper.dialogs;

import dsuyasov.recognition.rulesModel.*;
import dsuyasov.recognition.rulesModel.generators.LabelRemover;
import dsuyasov.recognition.rulesModel.checkers.ColorChecker;
import dsuyasov.recognition.rulesModel.checkers.LabelChecker;
import dsuyasov.recognition.ui.helper.panels.cellSettingsPanel.ColorCheckerSelecter;
import dsuyasov.recognition.ui.helper.panels.cellSettingsPanel.ChooserPanel;
import dsuyasov.recognition.util.MessageUtil;
import dsuyasov.recognition.util.localization.Localization;

import javax.swing.*;
import java.awt.FlowLayout;
import java.awt.HeadlessException;
import java.awt.event.*;
import java.util.List;
import java.util.LinkedList;

/**
 * @author Dmitriy Suyasov
 *         Date 16.04.2007
 */
public class CellOptionsDialog extends AbstractDialog {

    public static final String TITLE = Localization.get(Localization.CELL_OPTION_DIALOG_TITLE);

    private ColorCheckerSelecter colorCheckerSelecter;
    private List<LabelChecker> labelCheckers;

    private RuleCell ruleCell;

    public CellOptionsDialog(int line, int column) throws HeadlessException {
        
        JPanel mainPanel = new JPanel();
        mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));
        setContentPane(mainPanel);

        CellularAutomata cellularAutomata = (CellularAutomata) Memory.get(Memory.CURRENT_RULE);
        //Integer selectedIndex = (Integer) Memory.get(Memory.SELECTED_RULE);
        RuleItem ruleItem = cellularAutomata.getSelectedRuleItem();//(selectedIndex == 0) ? cellularAutomata.getDefaultRuleItem() : cellularAutomata.getRuleItems().get(selectedIndex - 1);
        this.ruleCell = ruleItem.getCell(line, column);

        colorCheckerSelecter = new ColorCheckerSelecter(this, mainPanel);
        colorCheckerSelecter.initialize(ruleCell.getColorChecker());

        labelCheckers = new LinkedList<LabelChecker>(ruleCell.getLabelCheckers());
        JPanel labelCheckersPabel = new JPanel();
        labelCheckersPabel.setBorder(BorderFactory.createTitledBorder( getTitle() ));
        labelCheckersPabel.setLayout(new BoxLayout(labelCheckersPabel, BoxLayout.Y_AXIS));
        mainPanel.add(labelCheckersPabel);

        JPanel labelCheckersSelecterPabel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        labelCheckersPabel.add(labelCheckersSelecterPabel);
        labelCheckersSelecterPabel.add(new JLabel(Localization.get(Localization.SELECT_CHECKER_FIELD_TITLE)));
        JList simbolsCheckersList = new JList(LabelChecker.getTitles());
        simbolsCheckersList.setBorder(BorderFactory.createEtchedBorder());
        labelCheckersSelecterPabel.add(simbolsCheckersList);
        JButton addLabelCheckerButton = new JButton(Localization.get(Localization.ADD_CHECKER_BUTTON_NAME));
        labelCheckersSelecterPabel.add(addLabelCheckerButton);

        JPanel simbolsCheckersListPanel = new JPanel();
        simbolsCheckersListPanel.setLayout(new BoxLayout(simbolsCheckersListPanel, BoxLayout.Y_AXIS));
        labelCheckersPabel.add(simbolsCheckersListPanel);
        LabelRemover labelRemover = ruleItem.getResult().getLabelRemover();
        addLabelCheckerButton.addActionListener(new AddLabelsCheckerListener(simbolsCheckersListPanel, simbolsCheckersList, labelRemover));
        drawLabelsCheckers(simbolsCheckersListPanel, labelRemover);

        drawOkCancelButtons(mainPanel, Localization.get(Localization.UPDATE_DIALOG_BUTTON_NAME),
                Localization.get(Localization.CANCEL_DIALOG_BUTTON_NAME));
        
        setResizable(false);

        setDefaultCloseOperation(
            JDialog.DO_NOTHING_ON_CLOSE);
        addWindowListener(new CellOptionDialogWindowListener(this));
    }

    private void drawLabelsCheckers(JPanel simbolsCheckersListPanel, LabelRemover labelRemover) {
        simbolsCheckersListPanel.removeAll();
        ChooserPanel chooserPanel = new ChooserPanel(simbolsCheckersListPanel, this);
        for (LabelChecker labelChecker : labelCheckers) {
            JButton removeChecker = new JButton(Localization.get(Localization.REMOVE_BUTTON_NAME));
            removeChecker.addActionListener(new RemoveCheckerListener(simbolsCheckersListPanel, labelChecker, labelRemover));
            labelChecker.draw(chooserPanel, false).add(removeChecker);
        }
        simbolsCheckersListPanel.revalidate();
        simbolsCheckersListPanel.repaint();
        pack();
    }

    public String getDialogTitle() {
        return TITLE;
    }

    private class RemoveCheckerListener implements ActionListener {

        private LabelChecker labelChecker;
        private JPanel labelCheckersPanel;
        private LabelRemover labelRemover;

        public RemoveCheckerListener(JPanel labelCheckersPabel, LabelChecker labelChecker, LabelRemover labelRemover) {
            this.labelChecker = labelChecker;
            this.labelRemover = labelRemover;
            this.labelCheckersPanel = labelCheckersPabel;
        }

        public void actionPerformed(ActionEvent e) {
            for (LabelChecker labelChecker : labelRemover.getSelectedSymbolCheckers()) {
                if (labelChecker == this.labelChecker) {
                    MessageUtil.alert(Localization.get(Localization.CANNOT_DELETE_SYMBOL_CHECKER_ALERT));
                    return;
                }
            }
            labelCheckers.remove(this.labelChecker);
            drawLabelsCheckers(this.labelCheckersPanel, this.labelRemover);
        }
    }

    private class AddLabelsCheckerListener implements ActionListener {

        private JList checkersList;
        private JPanel labelCheckersPanel;
        private LabelRemover labelRemover;

        public AddLabelsCheckerListener(JPanel symbolCheckersPabel, JList checkersList, LabelRemover labelRemover) {
            this.checkersList = checkersList;
            this.labelCheckersPanel = symbolCheckersPabel;
            this.labelRemover = labelRemover;
        }

        public void actionPerformed(ActionEvent e) {
            String value = (String) checkersList.getSelectedValue();
            labelCheckers.add(LabelChecker.getLabelCheckerByTitle(value));
            drawLabelsCheckers(this.labelCheckersPanel, this.labelRemover);
        }
    }

    public void updateAndClose() {
        ColorChecker colorChecker = (ColorChecker) colorCheckerSelecter.getResult();
        if (colorChecker != null) {
            this.ruleCell.setColorChecker(colorChecker);
        }
        if (labelCheckers != null) {
            this.ruleCell.setLabelCheckers(labelCheckers);
        }
        close();
    }

    private class CellOptionDialogWindowListener extends WindowAdapter {

        private CellOptionsDialog parent;

        public CellOptionDialogWindowListener(CellOptionsDialog parent) {
            this.parent = parent;
        }

        public void windowClosing(WindowEvent we) {
            int result = JOptionPane.showConfirmDialog(parent, Localization.get(Localization.ACCEPT_CHANGES_QUESTION), Localization.get(Localization.QUESTION_DIALOG_TITLE),
                    JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
            if (result == JOptionPane.CANCEL_OPTION)
                return;
            if (result == JOptionPane.YES_OPTION) {
                parent.updateAndClose();
            } else {
                parent.close();
            }
        }
    }
}
