package dsuyasov.recognition.ui.helper.dialogs;

import dsuyasov.recognition.rulesModel.labels.Labels;
import dsuyasov.recognition.rulesModel.labels.LabelType;
import dsuyasov.recognition.util.localization.Localization;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.List;
import java.util.LinkedList;

/**
 * @author Dmitriy Suyasov
 */
public class LabelsOptionDialog extends AbstractDialog {

    public static final String TITLE = Localization.get(Localization.CHOOSE_SYMBOLS_DIALOG_TITLE);

    private List<Labels> labels;
    private JPanel labelsPanel;
    private boolean isOnlyOneSymbolAcceptable;

    public LabelsOptionDialog(List<Labels> symbols, boolean isOnlyOneLabelAcceptable) throws HeadlessException {
        this.labels = new LinkedList<Labels>(symbols);
        this.isOnlyOneSymbolAcceptable = isOnlyOneLabelAcceptable;

        JPanel mainPanel = new JPanel();
        mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));

        JPanel buttonsPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 10));
        mainPanel.add(buttonsPanel);

        for (LabelType labelType : LabelType.values()) {
            JButton createLabel = new JButton(Localization.get(Localization.CREATE_DIALOG_BUTTON_NAME) + " " + labelType.getTitle());
            createLabel.addActionListener(new AddLabelListener(labelType));
            buttonsPanel.add(createLabel);
        }

        if (!isOnlyOneLabelAcceptable) {
            JButton removeLabel = new JButton(Localization.get(Localization.REMOVE_SYMBOL_BUTTON_NAME));
            removeLabel.addActionListener(new RemoveLabelListener());
            buttonsPanel.add(removeLabel);
        }

        labelsPanel = new JPanel();
        labelsPanel.setLayout(new BoxLayout(labelsPanel, BoxLayout.Y_AXIS));
        mainPanel.add(labelsPanel);
        drawLabelsPanels();

        drawUpdateCancelButtons(mainPanel);

        setResizable(false);
        setContentPane(mainPanel);
    }

    protected void updateAndClose() {
        setVisible(false);
    }

    protected void close() {
        labels = null;
        setVisible(false);
    }

    public List<Labels> getLabels() {
        return labels;
    }

    private void drawLabelsPanels() {
        labelsPanel.removeAll();
        for (Labels label : labels) {
            label.drawDialog(labelsPanel);
        }
        pack();
    }

    public String getDialogTitle() {
        return TITLE;
    }

    private class AddLabelListener implements ActionListener {

        private LabelType labelType;

        public AddLabelListener(LabelType labelType) {
            this.labelType = labelType;
        }

        public void actionPerformed(ActionEvent e) {
            if (isOnlyOneSymbolAcceptable) labels.clear();
            labels.add( labelType.getLabel() );
            drawLabelsPanels();
        }
    }

    private class RemoveLabelListener implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            if (!labels.isEmpty()) {
                labels.remove(labels.size() - 1);
                drawLabelsPanels();
            }
        }
    }

}
