package dsuyasov.recognition.ui.helper.dialogs;

import dsuyasov.recognition.rulesModel.Memory;
import dsuyasov.recognition.rulesModel.MatrixElement;
import dsuyasov.recognition.rulesModel.labels.Labels;
import dsuyasov.recognition.rulesModel.recognition.StudiesSet;
import dsuyasov.recognition.files.LoadFile;
import dsuyasov.recognition.files.TemplateLoader;
import dsuyasov.recognition.files.SaveFile;
import dsuyasov.recognition.files.TemplateSaver;
import dsuyasov.recognition.ui.helper.panels.DrawImagePanel;
import dsuyasov.recognition.ui.helper.LabelsChanges;
import dsuyasov.recognition.ui.helper.panels.SelectLabelsPanel;
import dsuyasov.recognition.util.MessageUtil;
import dsuyasov.recognition.util.localization.Localization;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.util.List;
import java.util.LinkedList;

/**
 * @author Dmitriy Suyasov
 */
public class LearnDialog extends AbstractDialog implements LabelsChanges {

    public static final String TITLE = Localization.get(Localization.LEARNING_DIALOG_TITLE);

    private JPanel mainPanel;
    private List<Labels> acceptSymbols;

    public LearnDialog() throws HeadlessException {
        mainPanel = new JPanel();
        mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));

        JPanel toMergeWithPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        JRadioButton toMergeWithRadio = new JRadioButton(Localization.get(Localization.LOAD_TEMPLATE_TO_MERGE_BUTTON_NAME));
        toMergeWithPanel.add(toMergeWithRadio);
        JTextField templateFileNameField = new JTextField(getTemplatesFileName(), 20);
        templateFileNameField.setEditable(false);
        toMergeWithPanel.add(templateFileNameField);
        JButton loadTemplate = new JButton(Localization.get(Localization.LOAD_TEMPLATE_BUTTON_NAME));
        loadTemplate.addActionListener(new LoadTemplateListener(templateFileNameField));
        toMergeWithPanel.add(loadTemplate);
        mainPanel.add(toMergeWithPanel);

        JPanel toNewTemplatePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        JRadioButton makeNewTemplatesRadio = new JRadioButton(Localization.get(Localization.MAKE_NEW_TEMPLATE_BUTTON_NAME));
        toNewTemplatePanel.add(makeNewTemplatesRadio);
        toNewTemplatePanel.add(new JLabel("x: "));
        JTextField xField = new JTextField("5", 20);
        toNewTemplatePanel.add(xField);
        toNewTemplatePanel.add(new JLabel("y: "));
        JTextField yField = new JTextField("5", 20);
        toNewTemplatePanel.add(yField);
        mainPanel.add(toNewTemplatePanel);

        acceptSymbols = new LinkedList<Labels>();
        if (Memory.get(Memory.LAYERS_LABELS) != null) {
            acceptSymbols.addAll((List) Memory.get(Memory.LAYERS_LABELS));
        }
        JPanel forSymbolsPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        forSymbolsPanel.add(new JLabel(Localization.get(Localization.SYMBOLS_TO_ACCEPT_FOR_LAYERS_FIELD_TITLE)));
        JPanel symbolsPanel = new SelectLabelsPanel(acceptSymbols, this, Memory.LAYERS_LABELS);
        forSymbolsPanel.add(symbolsPanel);
        mainPanel.add(forSymbolsPanel);

        ButtonGroup group = new ButtonGroup();
        group.add(toMergeWithRadio);
        group.add(makeNewTemplatesRadio);
        toMergeWithRadio.setSelected(true);

        JPanel startCancelButtonsPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        JButton startButton = new JButton(Localization.get(Localization.START_DIALOG_BUTTON_NAME));
        startButton.addActionListener(new StartListener(toMergeWithRadio, xField, yField));
        startCancelButtonsPanel.add(startButton);
        JButton cancelButton = new JButton(Localization.get(Localization.CANCEL_DIALOG_BUTTON_NAME));
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                close();
            }
        });
        startCancelButtonsPanel.add(cancelButton);
        mainPanel.add(startCancelButtonsPanel);

        setContentPane(mainPanel);
        mainPanel.revalidate();
        mainPanel.repaint();
        setResizable(false);
        pack();
        setVisible(true);
    }

    public String getDialogTitle() {
        return TITLE;
    }

    public void updateAndClose() {}

    public void updateLabels(List<Labels> symbols) {
        acceptSymbols = new LinkedList<Labels>(symbols);
        pack();
    }

    private class StartListener implements ActionListener {

        private JRadioButton toMergeWithRadio;
        private JTextField xField;
        private JTextField yField;

        public StartListener(JRadioButton toMergeWithRadio, JTextField xField, JTextField yField) {
            this.toMergeWithRadio = toMergeWithRadio;
            this.xField = xField;
            this.yField = yField;
        }

        public void actionPerformed(ActionEvent e) {
            if (acceptSymbols.size() == 0) {
                MessageUtil.alert(Localization.get(Localization.SELECT_SYMBOLS_ALERT));
                return;
            }
            if (!toMergeWithRadio.isSelected()) {
                try {
                    StudiesSet studiesSet = new StudiesSet(Integer.parseInt(xField.getText()), Integer.parseInt(yField.getText()));
                    Memory.add(Memory.TEMPLATE, studiesSet);
                } catch(NumberFormatException nfe) {
                    MessageUtil.alert(Localization.get(Localization.X_Y_SHOULD_BE_AN_INTEGER_ALERT));
                    return;
                }
            }
            redraw();
        }
    }

    private List<DrawImagePanel> panels;
    private List<MatrixElement[][]> elements;

    private int index;

    private JTextField textField;
    private JButton okButton;
    private JPanel imagePanel;

    private void redraw() {
        mainPanel.removeAll();

        panels = (List<DrawImagePanel>) Memory.get(Memory.IMAGES);
        elements = (List<MatrixElement[][]>) Memory.get(Memory.MATRIX);

        index = -1;
        imagePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        mainPanel.add(imagePanel);
        JPanel textPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        textField = new JTextField("", 20);
        textField.addKeyListener(new KeyListener() {
            public void keyTyped(KeyEvent e) {}

            public void keyPressed(KeyEvent e) {
                if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                    save();
                    toNext();
                }
            }

            public void keyReleased(KeyEvent e) {}
        });
        textPanel.add(textField);
        mainPanel.add(textPanel);

        JPanel okCancelPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        okButton = new JButton(Localization.get(Localization.NEXT_DIALOG_BUTTON_NAME));
        okCancelPanel.add(okButton);
        JButton skipButton = new JButton(Localization.get(Localization.SKIP_DIALOG_BUTTON_NAME));
        skipButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                toNext();
            }
        });
        okCancelPanel.add(skipButton);
        JButton cancel = new JButton(Localization.get(Localization.CANCEL_DIALOG_BUTTON_NAME));
        cancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                close();
            }
        });
        okCancelPanel.add(cancel);
        mainPanel.add(okCancelPanel);
        updateButtonText();
        toNext();

        mainPanel.revalidate();
        mainPanel.repaint();
        pack();
    }

    private void updateButtonText() {
        for (ActionListener actionListener : okButton.getActionListeners()) {
            okButton.removeActionListener(actionListener);
        }
        if (index >= panels.size()) {
            okButton.setText(Localization.get(Localization.SAVE_RESULTS_DIALOG_BUTTON_NAME));
            okButton.addActionListener(new SaveResultsListener());
        } else {
            okButton.setText(Localization.get(Localization.NEXT_DIALOG_BUTTON_NAME));
            okButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    save();
                    toNext();
                }
            });
        }
    }

    private void save() {
        // Save template
        if (textField.getText().length() == 0) {
            MessageUtil.alert(Localization.get(Localization.SET_ELEMENT_FIRST_ALERT));
        }

        StudiesSet studySet = (StudiesSet) Memory.get(Memory.TEMPLATE);
        studySet.study(elements.get(index), acceptSymbols, textField.getText());
        //toNext();
    }



    private void toNext() {
        // Set next
        index++;
        if (index >= panels.size()) {
            updateButtonText();
            return;
        }
        DrawImagePanel imagePanel1 = panels.get(index);
        imagePanel1.setScale(1);
        imagePanel1.setPreferredSize(new Dimension(imagePanel1.getWidth(), imagePanel1.getHeight()));
        imagePanel.removeAll();
        imagePanel.add(imagePanel1);
        textField.setText("");
        updateButtonText();
        mainPanel.revalidate();
        mainPanel.repaint();
        pack();
    }

    private class SaveResultsListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            SaveFile saveFile = new SaveFile(new TemplateSaver());
            if (!saveFile.save()) {
                MessageUtil.setMessage(Localization.get(Localization.ERROR_WHILE_SAVING_MESSAGE));
            }
            Memory.add(Memory.SAVING_TEMPLATES, saveFile.getFileName());
        }
    }
    private class LoadTemplateListener implements ActionListener {

        private JTextField fieldToUpdate;

        public LoadTemplateListener(JTextField fieldToUpdate) {
            this.fieldToUpdate = fieldToUpdate;
        }

        public void actionPerformed(ActionEvent e) {
            LoadFile loadFile = new LoadFile(new TemplateLoader());
            StudiesSet aSet = (StudiesSet) loadFile.load();
            String fileName = loadFile.getFileName();
            Memory.add(Memory.TEMPLATE, aSet);
            Memory.add(Memory.SAVING_TEMPLATES, fileName);
            fieldToUpdate.setText(fileName);
        }
    }

    private String getTemplatesFileName() {
        String element = (String) Memory.get(Memory.SAVING_TEMPLATES);
        if (element == null) {
            element = "";
            Memory.add(Memory.SAVING_TEMPLATES, element);
        }
        return element;
    }
}
