package dsuyasov.recognition.ui.page;

import dsuyasov.recognition.rulesModel.Memory;
import dsuyasov.recognition.rulesModel.CellularAutomata;
import dsuyasov.recognition.files.LoadException;
import dsuyasov.recognition.rulesModel.sequence.AutomataSequence;
import dsuyasov.recognition.rulesModel.sequence.SpecialElement;
import dsuyasov.recognition.files.*;
import dsuyasov.recognition.ui.helper.panels.AutomataSequenceWithSettingsPanel;
import dsuyasov.recognition.util.MessageUtil;
import dsuyasov.recognition.util.localization.Localization;
import dsuyasov.recognition.ui.helper.dialogs.ChooseSpecialRuleDialog;
import dsuyasov.recognition.ui.helper.dialogs.AutomataSequenceDescriptionDialog;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.FilenameFilter;
import java.util.List;
import java.util.LinkedList;

import com.sun.org.apache.xerces.internal.jaxp.DocumentBuilderFactoryImpl;

/**
 * User: Suyasov Dmitriy
 */
public class CombineAutomataPage extends Page {

    public static final String COMBINE_RULES_HELPER_NAME = "combineRulesHelperName";
    private static final String RULE_EXTENTION = "rul";

    public String getUniqueKey() {
        return COMBINE_RULES_HELPER_NAME;
    }

    public static void setAsCurrent() {
        setAsCurrent(COMBINE_RULES_HELPER_NAME);
    }

    private List<CellularAutomata> loadCellularAutomatas;

    private JList ruleList;
    private DefaultListModel ruleListModel;
    private AutomataSequenceWithSettingsPanel ruleSequencePanel;
    private JTextArea cellAutomataDescriptionLabel;

    public CombineAutomataPage() {
        initialize();
    }

    public CombineAutomataPage(Page previous, Page next) {
        super(previous, next);
        initialize();
    }

    private void initialize() {
        ruleListModel = new DefaultListModel();
        loadCellularAutomatas = new LinkedList<CellularAutomata>();
    }

    public void draw() {
        JPanel root = (JPanel) Memory.get(Memory.MAIN_PANEL);
        root.removeAll();

        JPanel mainPanel = new JPanel(new GridLayout(1, 3, 10, 10));
        root.add(mainPanel);

        JPanel rulesPanel = new JPanel();
        rulesPanel.setLayout(new BoxLayout(rulesPanel, BoxLayout.Y_AXIS));
        mainPanel.add(rulesPanel);

        JPanel addRulesToListPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        rulesPanel.add(addRulesToListPanel);
        JButton addRulesToListButton = new JButton(Localization.get(Localization.ADD_RULES_TO_LIST_BUTTON_NAME));
        addRulesToListButton.addActionListener(new ChooseFileAndDirectoryListener());
        addRulesToListPanel.add(addRulesToListButton);

        JPanel clearRulesListPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        rulesPanel.add(clearRulesListPanel);
        JButton clearRulesListButton = new JButton(Localization.get(Localization.CLEAR_RULES_LIST_BUTTON_NAME));
        clearRulesListButton.addActionListener(new ClearRulesListListener());
        clearRulesListPanel.add(clearRulesListButton);

        JPanel ruleListPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 10, 10));
        rulesPanel.add(ruleListPanel);
        ruleList = new JList(ruleListModel);

        ruleList.setBorder(BorderFactory.createEtchedBorder());
        ruleList.setPreferredSize(new Dimension(200, 400));
        ruleList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        ruleList.addMouseListener(new PopupListener());
        ruleListPanel.add(ruleList);

        JPanel addRulePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        rulesPanel.add(addRulePanel);
        JButton addRuleButton = new JButton(Localization.get(Localization.ADD_RULE_BUTTON_NAME));
        addRuleButton.addActionListener(new AddRuleToSequenceListener());
        addRulePanel.add(addRuleButton);

        JPanel addSpecialElementPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        rulesPanel.add(addSpecialElementPanel);
        JButton addSpecialElementButton = new JButton(Localization.get(Localization.ADD_SPECIAL_SYMBOL_BUTTON_NAME));
        addSpecialElementButton.addActionListener(new AddSpecialElementListener());
        addSpecialElementPanel.add(addSpecialElementButton);

        // Callular Automata and Cellular Automata Sequence descriptions

        if (this.ruleSequencePanel == null)
            this.ruleSequencePanel = new AutomataSequenceWithSettingsPanel();


        JPanel descriptionsPanel = new JPanel();
        descriptionsPanel.setLayout(new BoxLayout(descriptionsPanel, BoxLayout.Y_AXIS));
        mainPanel.add(descriptionsPanel);

        JPanel ruleDescriptionPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        cellAutomataDescriptionLabel = new JTextArea(Localization.get(Localization.CELLULAR_AUTOMATA_SEQUENCE_DESCRIPTION_MESSAGE) + ": " + AutomataSequence.getRulesSequence().getDescription());
        cellAutomataDescriptionLabel.setColumns(25);
        cellAutomataDescriptionLabel.setLineWrap(true);
        cellAutomataDescriptionLabel.setBackground(Color.LIGHT_GRAY);
        cellAutomataDescriptionLabel.setFont(new Font("Arial", Font.BOLD, 12));
        cellAutomataDescriptionLabel.setEditable(false);
        ruleDescriptionPanel.add(cellAutomataDescriptionLabel);

        JTextArea ruleDescriptionLabel = new JTextArea(
                ruleSequencePanel.getSelectedRule() < 0 ? Localization.get(Localization.CELLULAR_AUTOMATA_DESCRIPTION_MESSAGE) + ": " :
                        AutomataSequence.getRulesSequence().getRules().get(ruleSequencePanel.getSelectedRule()).getDescription()
        );
        this.ruleSequencePanel.setRuleDescription(ruleDescriptionLabel);
        ruleDescriptionLabel.setLineWrap(true);
        ruleDescriptionLabel.setEditable(false);
        ruleDescriptionLabel.setColumns(25);
        ruleDescriptionLabel.setBackground(Color.LIGHT_GRAY);
        ruleDescriptionLabel.setFont(new Font("Arial", Font.BOLD, 12));
        ruleDescriptionPanel.add(ruleDescriptionLabel);
        descriptionsPanel.add(ruleDescriptionPanel);

        // Cellular Automata Sequence Panels

        JPanel mainRuleSequencePanel = new JPanel();
        mainRuleSequencePanel.setLayout(new BoxLayout(mainRuleSequencePanel, BoxLayout.Y_AXIS));
        mainRuleSequencePanel.setMinimumSize(new Dimension(300, 500));
        mainRuleSequencePanel.setPreferredSize(new Dimension(300, 500));
        mainPanel.add(mainRuleSequencePanel);

        JPanel forRuleSequencePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        mainRuleSequencePanel.add(forRuleSequencePanel);

        // Make ruleSelection scroll panel
        this.ruleSequencePanel.scrollRectToVisible(this.ruleSequencePanel.getSizes());
        this.ruleSequencePanel.setPreferredSize(this.ruleSequencePanel.getSizes().getSize());
        JScrollPane ruleSequenceScrollPane = new JScrollPane(this.ruleSequencePanel, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        ruleSequenceScrollPane.setPreferredSize(new Dimension(280, 370));
        ruleSequenceScrollPane.setMinimumSize(new Dimension(280, 370));
        forRuleSequencePanel.add(ruleSequenceScrollPane);
        mainPanel.revalidate();
        mainPanel.repaint();

        JPanel sequenceButtonsPanel = new JPanel(new GridLayout(4, 2, 5, 5));
        mainRuleSequencePanel.add(sequenceButtonsPanel);

        JPanel cellAutomatShowDescriptionPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        sequenceButtonsPanel.add(cellAutomatShowDescriptionPanel);
        JButton cellAutomatShowDescriptionButton = new JButton(Localization.get(Localization.DESCRIPTION_BUTTON_NAME));
        cellAutomatShowDescriptionButton.setPreferredSize(new Dimension(130, 30));
        cellAutomatShowDescriptionButton.addActionListener(new CellAutomataSequenceDescriptionListener());
        cellAutomatShowDescriptionPanel.add(cellAutomatShowDescriptionButton);

        sequenceButtonsPanel.add(new JPanel());

        JPanel clearElementsPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        sequenceButtonsPanel.add(clearElementsPanel);
        JButton clearElementsButton = new JButton(Localization.get(Localization.CLEAR_SEQUENCE_BUTTON_NAME));
        clearElementsButton.setPreferredSize(new Dimension(130, 30));
        clearElementsButton.addActionListener(new ClearRulesSequenceListener());
        clearElementsPanel.add(clearElementsButton);

        JPanel removeElementPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        sequenceButtonsPanel.add(removeElementPanel);
        JButton removeElementButton = new JButton(Localization.get(Localization.REMOVE_ELEMENT_BUTTON_NAME));
        removeElementButton.setPreferredSize(new Dimension(130, 30));
        removeElementButton.addActionListener(new RemoveRuleFromSequenceListener());
        removeElementPanel.add(removeElementButton);

        JPanel moveUpElementPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        sequenceButtonsPanel.add(moveUpElementPanel);
        JButton moveUpElementButton = new JButton(Localization.get(Localization.MOVEUP_AUTOMATA_BUTTON_NAME));
        moveUpElementButton.setPreferredSize(new Dimension(130, 30));
        moveUpElementButton.addActionListener(new MoveUpRuleListener());
        moveUpElementPanel.add(moveUpElementButton);

        JPanel moveDownElementPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        sequenceButtonsPanel.add(moveDownElementPanel);
        JButton moveDownElementButton = new JButton(Localization.get(Localization.MOVEDOWN_AUTOMATA_BUTTON_NAME));
        moveDownElementButton.setPreferredSize(new Dimension(130, 30));
        moveDownElementButton.addActionListener(new MoveDownRuleListener());
        moveDownElementPanel.add(moveDownElementButton);

        JPanel saveSequencePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        sequenceButtonsPanel.add(saveSequencePanel);
        JButton saveSequenceButton = new JButton(Localization.get(Localization.SAVE_SEQUENCE_BUTTON_NAME));
        saveSequenceButton.setPreferredSize(new Dimension(130, 30));
        saveSequenceButton.addActionListener(new SaveSequenceListener());
        saveSequencePanel.add(saveSequenceButton);

        JPanel loadSequencePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        sequenceButtonsPanel.add(loadSequencePanel);
        JButton loadSequenceButton = new JButton(Localization.get(Localization.LOAD_SEQUENCE_BUTTON_NAME));
        loadSequenceButton.setPreferredSize(new Dimension(130, 30));
        loadSequenceButton.addActionListener(new LoadSequenceListener());
        loadSequencePanel.add(loadSequenceButton);
    }

    public String getDescription() {
        return Localization.get(Localization.COMBINE_RULES_PAGE);
    }

    private class PopupListener extends MouseAdapter {

        private JPopupMenu ruleListMenu = null;

        public void mousePressed(MouseEvent e) {
            maybeShowPopup(e);
        }

        public void mouseReleased(MouseEvent e) {
            maybeShowPopup(e);
        }

        private void maybeShowPopup(MouseEvent e) {

            if (ruleListMenu != null)
                ruleList.remove(ruleListMenu);
            

            if (ruleListModel.size() > 0 && e.isPopupTrigger() &&
                    ruleList.getCellBounds(0, ruleListModel.size() - 1).contains(e.getX(), e.getY())) {
                ruleListMenu = new JPopupMenu("menu");

                for (int iLoop = 0; iLoop < ruleListModel.size(); iLoop++) {
                    if (ruleList.getCellBounds(iLoop, iLoop).contains(e.getX(), e.getY())) {

                        JMenuItem addAsCurrentRuleMenuItem = new JMenuItem(Localization.get(Localization.ADD_AS_CURRENT_RULE_MENU_ITEM_NAME));
                        ruleListMenu.add(addAsCurrentRuleMenuItem);
                        addAsCurrentRuleMenuItem.addActionListener(new AddAsCurrentRuleListener(iLoop));

                        JMenuItem removeRuleMenuItem = new JMenuItem(Localization.get(Localization.REMOVE_RULE_FROM_LIST_MENU_ITEM_NAME));
                        ruleListMenu.add(removeRuleMenuItem);
                        removeRuleMenuItem.addActionListener(new RemoveRuleFromTheListListener(iLoop));

                        break;
                    }
                }

                ruleList.add(ruleListMenu);

                ruleListMenu.show(e.getComponent(),
                           e.getX(), e.getY());
            }
        }
    }

    private class AddAsCurrentRuleListener implements ActionListener {

        private int rileIndex;

        public AddAsCurrentRuleListener(int rileIndex) {
            this.rileIndex = rileIndex;
        }

        public void actionPerformed(ActionEvent e) {
            Memory.add(Memory.CURRENT_RULE, loadCellularAutomatas.get(rileIndex));
        }
    }

    private class RemoveRuleFromTheListListener implements ActionListener {

        private int ruleIndexToRemove;

        public RemoveRuleFromTheListListener(int ruleIndexToRemove) {
            this.ruleIndexToRemove = ruleIndexToRemove;
        }

        public void actionPerformed(ActionEvent e) {
            loadCellularAutomatas.remove(ruleIndexToRemove);
            ruleListModel.remove(ruleIndexToRemove);
        }
    }

    private void ruleSequenceRepaint() {
        ruleSequencePanel.scrollRectToVisible(ruleSequencePanel.getSizes());
        ruleSequencePanel.setPreferredSize(ruleSequencePanel.getSizes().getSize());
        ruleSequencePanel.revalidate();
        ruleSequencePanel.repaint();
    }

    private void ruleSequenceUpdateSelectionAndRepaint(int offset) {
        ruleSequencePanel.updateSelection(offset);
        ruleSequenceRepaint();
    }

    private class ClearRulesListListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            loadCellularAutomatas.clear();
            ruleListModel.clear();
        }
    }

    private class ChooseFileAndDirectoryListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            JFileChooser fc = new JFileChooser();
            fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
            SaveFile.RuleFileFilter filter = new SaveFile.RuleFileFilter();
            filter.addExtension(RULE_EXTENTION);
            filter.setDescription(Localization.get(Localization.RULE_FILES_AND_DIRECTORIES_MESSAGE));
            fc.setFileFilter(filter);
            int result = fc.showOpenDialog((JPanel) Memory.get(Memory.MAIN_PANEL));
            if (result == JFileChooser.APPROVE_OPTION) {
                File file = fc.getSelectedFile();
                File[] files;
                if (file.isDirectory()) {
                    files = file.listFiles(new FilenameFilter() {
                        public boolean accept(File dir, String name) {
                            return name.substring(name.lastIndexOf(".") + 1).equalsIgnoreCase(RULE_EXTENTION);
                        }
                    });
                } else {
                    files = new File[1];
                    files[0] = file;
                }
                for (File ruleFile : files) {
                    CellularAutomata newCellularAutomata;
                    try {
                        newCellularAutomata = new CellularAutomata(DocumentBuilderFactoryImpl.newInstance().newDocumentBuilder().parse(ruleFile));
                    } catch (LoadException ex) {
                        MessageUtil.setMessage(ex.getMessage());
                        return;
                    } catch (Exception ex) {
                        MessageUtil.setMessage(Localization.get(Localization.ERROR_WHILE_RULE_LOAD_MESSAGE));
                        return;
                    }
                    loadCellularAutomatas.add(newCellularAutomata);
                    ruleListModel.insertElementAt(newCellularAutomata.getName(), ruleListModel.size());
                }
            }
        }
    }

    private class AddRuleToSequenceListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            int index = ruleList.getSelectedIndex();
            if (index >= 0) {
                AutomataSequence.getRulesSequence().addRule(loadCellularAutomatas.get(index));
                ruleSequenceRepaint();
            }
        }
    }

    private class RemoveRuleFromSequenceListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            int index = ruleSequencePanel.getSelectedRule();
            if (index >= 0) {
                if (AutomataSequence.getRulesSequence().canBeRemoved(index)) {
                    AutomataSequence.getRulesSequence().removeRule(index);
                    ruleSequenceUpdateSelectionAndRepaint(0);
                } else MessageUtil.alert(Localization.get(Localization.ELEMENT_CANNOT_BE_REMOVED_ALERT));
            }
        }
    }

    private class MoveUpRuleListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            int index = ruleSequencePanel.getSelectedRule();
            if (index >= 0) {
                AutomataSequence.getRulesSequence().moveRule(index, -1);
                ruleSequenceUpdateSelectionAndRepaint(-1);
            }
        }
    }

    private class MoveDownRuleListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            int index = ruleSequencePanel.getSelectedRule();
            if (index >= 0) {
                AutomataSequence.getRulesSequence().moveRule(index, 1);
                ruleSequenceUpdateSelectionAndRepaint(1);
            }
        }
    }

    private class CellAutomataSequenceDescriptionListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            new AutomataSequenceDescriptionDialog();
            cellAutomataDescriptionLabel.setText(Localization.get(
                    Localization.CELLULAR_AUTOMATA_SEQUENCE_DESCRIPTION_MESSAGE) + ": " +
                    AutomataSequence.getRulesSequence().getDescription());
            ((JPanel) Memory.get(Memory.MAIN_PANEL)).revalidate();
            ((JPanel) Memory.get(Memory.MAIN_PANEL)).repaint();
        }
    }

    private class ClearRulesSequenceListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            AutomataSequence.getRulesSequence().getRules().clear();
            ruleSequenceUpdateSelectionAndRepaint(0);
        }
    }

    private class AddSpecialElementListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            ChooseSpecialRuleDialog chooseSpecialRuleDialog = new ChooseSpecialRuleDialog();

            SpecialElement element = chooseSpecialRuleDialog.getSelectedElement();
            if (element != null) {
                AutomataSequence.getRulesSequence().addRule(element);
                ruleSequenceRepaint();
            }
        }
    }

    private class SaveSequenceListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            new SaveFile(new CellularAutomataSequenceSaver(AutomataSequence.getRulesSequence())).save();
        }
    }

    private class LoadSequenceListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            AutomataSequence.setRulesSequence((AutomataSequence) new LoadFile(new CellularAutomataSequenceLoader()).load());
            ruleSequenceRepaint();
        }
    }

}
