package dsuyasov.recognition.ui.page;

import dsuyasov.recognition.rulesModel.Memory;
import dsuyasov.recognition.rulesModel.CellularAutomata;
import dsuyasov.recognition.ui.helper.panels.AutomataSelectionPanel;
import dsuyasov.recognition.ui.helper.panels.AutomataDescriptionPanel;
import dsuyasov.recognition.files.*;
import dsuyasov.recognition.ui.helper.dialogs.AutomataSettingsDialog;
import dsuyasov.recognition.ui.helper.dialogs.NewAutomataSettingsDialog;
import dsuyasov.recognition.util.MessageUtil;
import dsuyasov.recognition.util.localization.Localization;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/**
 * @author Dmitriy Suyasov
 *         Date 09.04.2007
 */
public class ShowAutomataPage extends Page {

    public static final String SHOW_RULES_HELPER_NAME = "showRulesHelperName";

    private JTextArea ruleDescriptionLabel;

    public String getUniqueKey() {
        return SHOW_RULES_HELPER_NAME;
    }

    public void draw() {

        JPanel root = (JPanel) Memory.get(Memory.MAIN_PANEL);
        root.removeAll();

        JPanel mainPanel = new JPanel(new BorderLayout(2, 2));
        root.add(mainPanel);

        JPanel rulesPanel = new JPanel(new BorderLayout());

        CellularAutomata currentCellularAutomata = (CellularAutomata) Memory.get(Memory.CURRENT_RULE);
        if (currentCellularAutomata == null) {
            rulesPanel.add(new JLabel(Localization.get(Localization.PLEASE_LOAD_THE_RULE_MESSAGE)), BorderLayout.CENTER);
        } else {
            // Make panels
            JPanel ruleSelectionPanel = new JPanel(new BorderLayout(5, 5));
            rulesPanel.add(ruleSelectionPanel, BorderLayout.EAST);
            JPanel ruleSelectionScrollPanel = new JPanel(new BorderLayout(5, 5));
            ruleSelectionPanel.add(ruleSelectionScrollPanel, BorderLayout.CENTER);
            JPanel ruleSelectionButtonsPanel = new JPanel(new GridLayout(4, 1, 5, 5));
            ruleSelectionPanel.add(ruleSelectionButtonsPanel, BorderLayout.SOUTH);

            // Make buttons
            JButton addNewRuleButton = new JButton(Localization.get(Localization.ADD_RULE_BUTTON_NAME));
            addNewRuleButton.addActionListener(new AddRuleListener());
            ruleSelectionButtonsPanel.add(addNewRuleButton);
            JButton removeRuleButton = new JButton(Localization.get(Localization.REMOVE_RULE_BUTTON_NAME));
            removeRuleButton.addActionListener(new RemoveRuleListener());
            ruleSelectionButtonsPanel.add(removeRuleButton);
            JButton moveRuleUpButton = new JButton(Localization.get(Localization.MOVEUP_RULE_BUTTON_NAME));
            moveRuleUpButton.addActionListener(new MoveRuleListener(-1));
            ruleSelectionButtonsPanel.add(moveRuleUpButton);
            JButton moveRuleDownButton = new JButton(Localization.get(Localization.MOVEDOWN_RULE_BUTTON_NAME));
            moveRuleDownButton.addActionListener(new MoveRuleListener(1));
            ruleSelectionButtonsPanel.add(moveRuleDownButton);

            // Make ruleSelection scroll panel
            AutomataSelectionPanel automataSelectionGraphicPanel = (AutomataSelectionPanel)Memory.get(Memory.RULE_SELECTION_PANEL);
            if (automataSelectionGraphicPanel == null)
                automataSelectionGraphicPanel = new AutomataSelectionPanel();
            Memory.add(Memory.RULE_SELECTION_PANEL, automataSelectionGraphicPanel);
            automataSelectionGraphicPanel.scrollRectToVisible(automataSelectionGraphicPanel.getSizes());
            automataSelectionGraphicPanel.setPreferredSize(automataSelectionGraphicPanel.getSizes().getSize());
            JScrollPane ruleSelectionScrollPane = new JScrollPane(automataSelectionGraphicPanel, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
            ruleSelectionScrollPane.setPreferredSize(new Dimension(148, 500));
            ruleSelectionScrollPanel.add(ruleSelectionScrollPane);
            automataSelectionGraphicPanel.revalidate();
            automataSelectionGraphicPanel.repaint();

            // Make main panel
            JPanel mainRulePanel = new JPanel(new BorderLayout(10, 10));
            rulesPanel.add(mainRulePanel, BorderLayout.CENTER);

            JPanel ruleSettingsPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
            mainRulePanel.add(ruleSettingsPanel, BorderLayout.NORTH);
            JLabel ruleName = new JLabel(Localization.get(Localization.RULE_TITLE) + currentCellularAutomata.getName());
            Memory.add(Memory.RULE_NAME_LABEL, ruleName);
            ruleSettingsPanel.add(ruleName);
            JButton ruleSettingsButton = new JButton(Localization.get(Localization.RULE_SETTINGS_BUTTON_NAME));
            ruleSettingsButton.addActionListener(new RuleSettingsListener());
            ruleSettingsPanel.add(ruleSettingsButton);

            JPanel ruleItemAndDescriptionPanel = new JPanel();
            ruleItemAndDescriptionPanel.setLayout(new BoxLayout(ruleItemAndDescriptionPanel, BoxLayout.Y_AXIS));
            
            JPanel ruleItemPanel = new JPanel();
            Memory.add(Memory.RULE_ITEM_PANEL, new AutomataDescriptionPanel(ruleItemPanel));
            ruleItemAndDescriptionPanel.add(ruleItemPanel);

            JPanel ruleDescriptionPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
            ruleDescriptionLabel = new JTextArea(currentCellularAutomata.getDescription());
            ruleDescriptionLabel.setEditable(false);
            ruleDescriptionLabel.setColumns(40);
            ruleDescriptionLabel.setLineWrap(true);
            ruleDescriptionPanel.add(ruleDescriptionLabel);
            
            mainRulePanel.add(ruleItemAndDescriptionPanel, BorderLayout.WEST);
            mainRulePanel.add(ruleDescriptionPanel, BorderLayout.SOUTH);
        }

        mainPanel.add(rulesPanel, BorderLayout.CENTER);

        JPanel rulesButtonsPanel = new JPanel();
        rulesButtonsPanel.setLayout(new BoxLayout(rulesButtonsPanel, BoxLayout.LINE_AXIS));
        rulesButtonsPanel.applyComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
        mainPanel.add(rulesButtonsPanel, BorderLayout.SOUTH);

        JButton loadRuleButton = new JButton(Localization.get(Localization.LOAD_RULE_BUTTON_NAME));
        loadRuleButton.addActionListener(new LoadRuleListener());
        rulesButtonsPanel.add(loadRuleButton);
        JButton saveRuleButton = new JButton(Localization.get(Localization.SAVE_RULE_BUTTON_NAME));
        saveRuleButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                CellularAutomata cellularAutomata = (CellularAutomata) Memory.get(Memory.CURRENT_RULE);
                if (cellularAutomata != null) {
                    if (!new SaveFile(new CellularAutomataSaver(cellularAutomata)).save()) {
                        MessageUtil.setMessage(Localization.get(Localization.SOMETHING_WRONG_WHILE_SAVING_MESSAGE));
                    }
                }
            }
        });
        rulesButtonsPanel.add(saveRuleButton);
        JButton newRuleButton = new JButton(Localization.get(Localization.NEW_RULE_BUTTON_NAME));
        newRuleButton.addActionListener(new NuwRuleListener());
        rulesButtonsPanel.add(newRuleButton);

        root.setVisible(false);
        root.setVisible(true);
    }

    public String getDescription() {
        return Localization.get(Localization.SHOW_RULES_PAGE);
    }

    private class LoadRuleListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            Object ruleObj = new LoadFile(new CellularAutomataLoader()).load();
            if (ruleObj != null) {
                Memory.add(Memory.CURRENT_RULE, ruleObj);
            }
            setAsCurrent();
        }
    }

    private class RuleSettingsListener implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            AutomataSettingsDialog dialog = new AutomataSettingsDialog();
            dialog.pack();
            dialog.setVisible(true);

            JPanel root = (JPanel) Memory.get(Memory.MAIN_PANEL);
            JLabel ruleName = (JLabel) Memory.get(Memory.RULE_NAME_LABEL);
            CellularAutomata cellularAutomata =(CellularAutomata) Memory.get(Memory.CURRENT_RULE);
            ruleName.setText(Localization.get(Localization.RULE_TITLE) + cellularAutomata.getName());
            ruleDescriptionLabel.setText(cellularAutomata.getDescription());
            root.repaint();
        }
    }

    private static class NuwRuleListener implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            JDialog newRuleSittingsDialog = new NewAutomataSettingsDialog();
            Rectangle appletRect = ((JApplet) Memory.get(Memory.APPLET)).getBounds();
            int dialogX = appletRect.getLocation().x + (appletRect.width - NewAutomataSettingsDialog.WIDTH) / 2;
            int dialogY = appletRect.getLocation().y + (appletRect.height - NewAutomataSettingsDialog.HEIGHT) / 2;
            newRuleSittingsDialog.setBounds(new Rectangle(dialogX, dialogY, NewAutomataSettingsDialog.WIDTH,
                    NewAutomataSettingsDialog.HEIGHT));
            newRuleSittingsDialog.pack();
            newRuleSittingsDialog.setVisible(true);
            JPanel ruleItemPanel = (JPanel) Memory.get(Memory.RULE_ITEM_PANEL);
            if (ruleItemPanel != null) ruleItemPanel.repaint();
        }
    }

    public static void setAsCurrent() {
        setAsCurrent(SHOW_RULES_HELPER_NAME);
    }

    private class AddRuleListener implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            AutomataSelectionPanel automataSelectionGraphicPanel = (AutomataSelectionPanel) Memory.get(Memory.RULE_SELECTION_PANEL);
            CellularAutomata cellularAutomata = (CellularAutomata) Memory.get(Memory.CURRENT_RULE);
            cellularAutomata.addNewRule();
            MessageUtil.setMessage(Localization.get(Localization.RULE_TITLE) + cellularAutomata.getRuleItems().size() + " " +
                    Localization.get(Localization.CREATED_TITLE));
            Memory.add(Memory.SELECTED_RULE, cellularAutomata.getRuleItems().size());
            automataSelectionGraphicPanel.scrollRectToVisible(automataSelectionGraphicPanel.getSizes());
            automataSelectionGraphicPanel.setPreferredSize(automataSelectionGraphicPanel.getSizes().getSize());
            automataSelectionGraphicPanel.revalidate();
            automataSelectionGraphicPanel.repaint();
            ((JPanel) Memory.get(Memory.RULE_ITEM_PANEL)).repaint();
        }

    }

    private class RemoveRuleListener implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            AutomataSelectionPanel automataSelectionGraphicPanel = (AutomataSelectionPanel) Memory.get(Memory.RULE_SELECTION_PANEL);
            CellularAutomata cellularAutomata = (CellularAutomata) Memory.get(Memory.CURRENT_RULE);
            Integer selectedRule = (Integer) Memory.get(Memory.SELECTED_RULE);
            if (selectedRule == 0) {
                JOptionPane.showMessageDialog(automataSelectionGraphicPanel,
                        Localization.get(Localization.CANNOT_REMOVE_ROOT_RULE_MESSAGE),
                        Localization.get(Localization.WARNING_DIALOG_TITLE), JOptionPane.WARNING_MESSAGE);
                return;
            } else if (selectedRule < 0) {
                JOptionPane.showMessageDialog(automataSelectionGraphicPanel,
                        Localization.get(Localization.SELECT_RULE_BEFORE_REMOVING_MESSAGE),
                        Localization.get(Localization.WARNING_DIALOG_TITLE), JOptionPane.WARNING_MESSAGE);
                return;
            }
            cellularAutomata.removeRule(selectedRule - 1);
            MessageUtil.setMessage(Localization.get(Localization.RULE_TITLE) + selectedRule + " " + Localization.get(Localization.REMOVED_TITLE));
            if (selectedRule > cellularAutomata.getRuleItems().size()) {
                Memory.add(Memory.SELECTED_RULE, selectedRule - 1);
            }
            automataSelectionGraphicPanel.scrollRectToVisible(automataSelectionGraphicPanel.getSizes());
            automataSelectionGraphicPanel.setPreferredSize(automataSelectionGraphicPanel.getSizes().getSize());
            automataSelectionGraphicPanel.revalidate();
            automataSelectionGraphicPanel.repaint();
            ((JPanel) Memory.get(Memory.RULE_ITEM_PANEL)).repaint();
        }

    }

    private class MoveRuleListener implements ActionListener {

        private int shift;

        public MoveRuleListener(int shift) {
            this.shift = shift;
        }

        public void actionPerformed(ActionEvent e) {
            AutomataSelectionPanel automataSelectionGraphicPanel = (AutomataSelectionPanel) Memory.get(Memory.RULE_SELECTION_PANEL);
            CellularAutomata cellularAutomata = (CellularAutomata) Memory.get(Memory.CURRENT_RULE);
            Integer selectedRule = (Integer) Memory.get(Memory.SELECTED_RULE);
            if (selectedRule < 0) {
                JOptionPane.showMessageDialog(automataSelectionGraphicPanel, Localization.get(Localization.SELECT_RULE_BEFORE_MOVING_MESSAGE),
                        Localization.get(Localization.WARNING_DIALOG_TITLE), JOptionPane.WARNING_MESSAGE);
                return;
            } else if (selectedRule == 0 || selectedRule + shift == 0) {
                JOptionPane.showMessageDialog(automataSelectionGraphicPanel,
                        Localization.get(Localization.CANNOT_MOVE_DEFAULT_RULE_MESSAGE),
                        Localization.get(Localization.WARNING_DIALOG_TITLE), JOptionPane.WARNING_MESSAGE);
                return;
            }
            int max = Math.max(selectedRule, selectedRule + shift);
            if (max > cellularAutomata.getRuleItems().size()) {
                return;
            }
            cellularAutomata.ruleInterchangeDown(max - 2);
            MessageUtil.setMessage(Localization.get(Localization.RULE_TITLE) + selectedRule + " " +
                    Localization.get(Localization.MOVED_TITLE) + " " +
                    (shift > 0 ? Localization.get(Localization.DOWN_TITLE) : Localization.get(Localization.UP_TITLE)));
            Memory.add(Memory.SELECTED_RULE, selectedRule + shift);
            automataSelectionGraphicPanel.scrollRectToVisible(automataSelectionGraphicPanel.getSizes());
            automataSelectionGraphicPanel.setPreferredSize(automataSelectionGraphicPanel.getSizes().getSize());
            automataSelectionGraphicPanel.revalidate();
            automataSelectionGraphicPanel.repaint();
        }
    }
}
