package dsuyasov.recognition.ui.page;

import dsuyasov.recognition.rulesModel.Memory;
import dsuyasov.recognition.rulesModel.labels.PredefinedLabels;
import dsuyasov.recognition.rulesModel.labels.Labels;
import dsuyasov.recognition.rulesModel.sequence.AutomataSequence;
import dsuyasov.recognition.rulesModel.sequence.OutputerElement;
import dsuyasov.recognition.rulesModel.sequence.Processable;
import dsuyasov.recognition.files.LoadFile;
import dsuyasov.recognition.files.CellularAutomataSequenceLoader;
import dsuyasov.recognition.util.MessageUtil;
import dsuyasov.recognition.util.localization.Localization;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/**
 * @author Dmitriy Suyasov
 */
public class StartAutomataSequencePage extends Page {

    public static final String START_RULES_HELPER_NAME = "startRulesHelperName";

    private JPanel startSequencePanel;
    private String directory;

    public StartAutomataSequencePage() {
        directory = "";
    }

    public String getUniqueKey() {
        return START_RULES_HELPER_NAME;
    }

    public static void setAsCurrent() {
        setAsCurrent(START_RULES_HELPER_NAME);
    }

    public void draw() {
        JPanel root = (JPanel) Memory.get(Memory.MAIN_PANEL);
        root.removeAll();

        JPanel mainPanel = new JPanel(new BorderLayout(5, 5));
        root.add(mainPanel);

        JPanel ruleSequenceLoader = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        ruleSequenceLoader.setBorder(BorderFactory.createTitledBorder(Localization.get(Localization.RULE_SEQUENCE_LOADING_TITLE)));
        JButton loadCurrentRuleSequence = new JButton(Localization.get(Localization.LOAD_CURRENT_RULE_SEQUENCE_BUTTON_NAME));
        loadCurrentRuleSequence.addActionListener(new LoadCurrentRuleSequenceListener());
        ruleSequenceLoader.add(loadCurrentRuleSequence);
        JButton loadRuleSequence = new JButton(Localization.get(Localization.LOAD_SEQUENCE_BUTTON_NAME));
        loadRuleSequence.addActionListener(new LoadRuleSequenceListener());
        ruleSequenceLoader.add(loadRuleSequence);
        JTextField directoryField = new JTextField(directory, 40);
        directoryField.setEditable(false);
        ruleSequenceLoader.add(directoryField);
        mainPanel.add(ruleSequenceLoader, BorderLayout.NORTH);

        if (startSequencePanel == null) {
            startSequencePanel = new JPanel();
        }
        mainPanel.add(startSequencePanel, BorderLayout.CENTER);

    }

    public String getDescription() {
        return Localization.get(Localization.START_RULE_SEQUENCES_PAGE);
    }

    private boolean checkImage() {
        BufferedImage image = (BufferedImage) Memory.get(Memory.PICTURE);
        if (image == null) {
            MessageUtil.alert(Localization.get(Localization.PLEASE_LOAD_PICTURE_BEFORE_PROCESSING_MESSAGE));
        }
        return image != null;
    }

    private void loadRuleSequence(AutomataSequence ruleSequence) {
        AutomataSequence newAutomataSequence = new AutomataSequence(ruleSequence);

        // Remove unused Outputter
        OutputerElement element = null;
        for (int iLoop = 0; iLoop < newAutomataSequence.getRules().size(); iLoop++) {
            Processable processable = newAutomataSequence.getRules().get(iLoop);
            if (processable.getLocalizationTitle().equals(OutputerElement.LOCALIZATION_TITLE)) {
                element = (OutputerElement) newAutomataSequence.removeRule(iLoop--);
            }
        }

        if (element == null) {
            element = new OutputerElement();
            for (Labels symbol : PredefinedLabels.getLabels()) {
                element.addLabel(symbol);
            }
        }

        newAutomataSequence.addRule(element);
        
        element.initialize(startSequencePanel, newAutomataSequence);
    }

    private void updatePanels() {
        ((JPanel) Memory.get(Memory.MAIN_PANEL)).revalidate();
        ((JPanel) Memory.get(Memory.MAIN_PANEL)).repaint();
    }

    private class LoadCurrentRuleSequenceListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            if (!checkImage()) {
                return;
            }
            directory = "";
            loadRuleSequence((AutomataSequence) Memory.get(Memory.CURRENT_SEQUENCE));
            updatePanels();
        }
    }

    private class LoadRuleSequenceListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            if (!checkImage()) {
                return;
            }
            LoadFile loader = new LoadFile(new CellularAutomataSequenceLoader());
            AutomataSequence loadedRuleSequence = (AutomataSequence) loader.load();
            if (loadedRuleSequence != null) {
                directory = loader.getFileName();
                loadRuleSequence(loadedRuleSequence);
            }
            updatePanels();
        }
    }

}
