package dsuyasov.recognition.util;

import org.w3c.dom.*;

import java.util.List;
import java.util.LinkedList;
import java.awt.*;

import dsuyasov.recognition.files.LoadException;

/**
 * @author Dmitriy Suyasov
 *         Date 17.04.2007
 */
public class XMLUtil {

    public static Node addNode(Document doc, Node parent, String name) {
        Element element = doc.createElement(name);
        parent.appendChild(element);
        return element;
    }

    public static void addAttribute(Document doc, Node parent, String name, String value) {
        Attr attr = doc.createAttribute(name);
        attr.setValue(value);
        parent.getAttributes().setNamedItem(attr);
    }

    public static void addCDATASection(Document doc, Node parent, String value) {
        parent.appendChild(doc.createCDATASection(value == null ? "" : value));
    }

    public static List<Node> getNode(Node parent, String name) {
        List<Node> result = new LinkedList<Node>();
        for (int iLoop = 0; iLoop < parent.getChildNodes().getLength(); iLoop++) {
            Node node = parent.getChildNodes().item(iLoop);
            if (node.getNodeName().equals(name)) {
                result.add(node);
            }
        }
        return result;
    }

    public static Node getSingleNode(Node parent, String name) {
        List<Node> result = getNode(parent, name);
        if (result == null || result.size() != 1)
            throw new RuntimeException("There are more than one (or none) of [" + name + "] nodes");
        return result.get(0);
    }

    public static String getNodeValue(Node parent) {
        for (int iLoop = 0; iLoop < parent.getChildNodes().getLength(); iLoop++) {
            Node node = parent.getChildNodes().item(iLoop);
            if (node.getNodeType() == Node.CDATA_SECTION_NODE) {
                return node.getNodeValue();
            }
        }
        for (int iLoop = 0; iLoop < parent.getChildNodes().getLength(); iLoop++) {
            Node node = parent.getChildNodes().item(iLoop);
            if (node.getNodeType() == Node.TEXT_NODE) {
                return node.getNodeValue();
            }
        }
        return null;
    }

    public static String getAttribute(Node parent, String name) {
        for (int iLoop = 0; iLoop < parent.getAttributes().getLength(); iLoop++) {
            Attr node = (Attr) parent.getAttributes().item(iLoop);
            if (node.getName().equals(name)) {
                return node.getValue();
            }
        }
        return null;
    }

    public static String getRequiredAttribute(Node parent, String name) throws LoadException {
        String result = getAttribute(parent, name);
        if (result == null)
            throw new LoadException("Attribute [" + name + "] unspecified");
        return result;
    }

    public static int getIntAttribute(Node parent, String name) throws LoadException {
        String attr = getRequiredAttribute(parent, name);
        try {
            return Integer.parseInt(attr);
        } catch(NumberFormatException nfe) {
            throw new LoadException("[" + name + "] unspecified");
        }
    }

    public static boolean getBooleanAttribute(Node parent, String name) throws LoadException {
        String attr = getRequiredAttribute(parent, name);
        try {
            return Boolean.parseBoolean(attr);
        } catch(NumberFormatException nfe) {
            throw new LoadException("[" + name + "] unspecified");
        }
    }

    public static final String RED_ATTR_NAME = "red";
    public static final String GREEN_ATTR_NAME = "green";
    public static final String BLUE_ATTR_NAME = "blue";
    public static final String ALPHA_ATTR_NAME = "alpha";

    public static void setColor(Document doc, Node node, Color color) {
        addAttribute(doc, node, RED_ATTR_NAME, String.valueOf(color.getRed()));
        addAttribute(doc, node, GREEN_ATTR_NAME, String.valueOf(color.getGreen()));
        addAttribute(doc, node, BLUE_ATTR_NAME, String.valueOf(color.getBlue()));
        addAttribute(doc, node, ALPHA_ATTR_NAME, String.valueOf(color.getAlpha()));
    }

    public static Color getColor(Node node) throws LoadException {
        return new Color(Integer.parseInt(getRequiredAttribute(node, RED_ATTR_NAME)),
                Integer.parseInt(getRequiredAttribute(node, GREEN_ATTR_NAME)),
                Integer.parseInt(getRequiredAttribute(node, BLUE_ATTR_NAME)),
                Integer.parseInt(getRequiredAttribute(node, ALPHA_ATTR_NAME)));
    }


}
